#!/bin/bash
# GxETH ARM variant

srcdir=/tmp/update
kernver="`uname -r`"
modver="`echo $kernver | cut -d- -f1`"
oldver="`modinfo -F version micam`"

# check files for zero size
for f in "$srcdir"/* "$srcdir"/web/*; do
  if [ ! -s $f ]; then
    exit 151
  fi
done

# install
install -m755 "$srcdir"/micamd /usr/bin
if [ $? -ne 0 ]; then exit 152; fi

install -m755 "$srcdir"/micamd-factory /usr/bin
if [ $? -ne 0 ]; then exit 153; fi

install -m4755 -g http "$srcdir"/micamd-cli /usr/bin
if [ $? -ne 0 ]; then exit 154; fi

install -m644 -t /srv/http "$srcdir"/web/*
if [ $? -ne 0 ]; then exit 155; fi

install -m644 -T "$srcdir"/micam.ko-"$modver" /usr/lib/modules/"$kernver"/extra/micam.ko
if [ $? -ne 0 ]; then exit 156; fi

# check for micamd-factory
if [ ! -e '/etc/systemd/system/micamd-factory.service' ]; then
  install -m644 "$srcdir"/micamd-factory.service /etc/systemd/system
  if [ $? -ne 0 ]; then exit 157; fi
  systemctl enable micamd-factory.service
  if [ $? -ne 0 ]; then exit 158; fi
fi

# check for wifi support
if [ ! -e '/etc/netctl/wlan0.dhcp.bak' ]; then
  install -m644 "$srcdir"/wlan0.dhcp.bak /etc/netctl
  install -m644 -T "$srcdir"/wlan0.dhcp.bak /etc/netctl/wlan0.dhcp
  if [ $? -ne 0 ]; then exit 159; fi

  install -m644 "$srcdir"/wlan0.static.bak /etc/netctl
  install -m644 -T "$srcdir"/wlan0.static.bak /etc/netctl/wlan0.static
  if [ $? -ne 0 ]; then exit 160; fi

  pacman -U "$srcdir"/libnl-3.2.28-1-armv7h.pkg.tar.xz --noconfirm
  if [ $? -ne 0 ]; then exit 161; fi
  pacman -U "$srcdir"/wireless_tools-30.pre9-1-armv7h.pkg.tar.xz --noconfirm
  if [ $? -ne 0 ]; then exit 162; fi
  pacman -U "$srcdir"/wpa_supplicant-1-2.6-2-armv7h.pkg.tar.xz --noconfirm
  if [ $? -ne 0 ]; then exit 163; fi
fi

# check for read only fs support
if [ ! -e '/usr/bin/write-enable' ]; then
  install -m755 "$srcdir"/write-enable /usr/bin
  if [ $? -ne 0 ]; then exit 164; fi
  install -m755 "$srcdir"/write-disable /usr/bin
  if [ $? -ne 0 ]; then exit 165; fi
fi

# check for serial support in config
if ! grep "serial_port_a" /etc/micamd.conf > /dev/null; then
  echo "" >> /etc/micamd.conf
  echo "serial_port_a = 49000" >> /etc/micamd.conf
  echo "serial_port_b = 49001" >> /etc/micamd.conf
fi

# check for lighttpd fix #1
if [ ! -e '/etc/lighttpd/.fix001' ]; then
  touch /etc/lighttpd/.fix001
  sed -i -e 's/php-fastcgi\.socket\",/php-fastcgi\.socket\" + var.PID,/' /etc/lighttpd/lighttpd.conf
fi

# check for OpenMP support
if [ ! -e '/usr/include/omp.h' ]; then
  pacman -U "$srcdir"/openmp-5.0.1-1-armv7h.pkg.tar.xz --noconfirm
  if [ $? -ne 0 ]; then exit 167; fi
fi

# load driver
depmod -A && modprobe micam
if [ $? -ne 0 ]; then
  exit 200
fi

# write changes to disk
sync

exit 0
