<?php

const TMP_LOCATION  = "/tmp";
const UPD_FILE_NAME = "micamd.tar.xz";

///////////////////////////////////////////////////////////////////////////////

function IsARM()
{
  return php_uname("m") == "armv7l";
}

function IsAstro()
{
  return GetVersionStr() == "astro";
}

function IsLANConnected()
{
  $state = trim(shell_exec("ip link list eth0 | grep -io \"state\s\+down\""));
  $carrier = trim(shell_exec("ip link list eth0 | grep -io \"NO-CARRIER\""));
  return $state === "" && $carrier === "";
}

function GetIP($wifi = false)
{
  $dev = $wifi ? "wlan0" : "eth0";
  $res = shell_exec("ip addr show $dev | grep -E 'inet [0-9]' | awk '{print $2}'");
  return trim(explode("/", $res)[0]);
}

function GetNetmask($wifi = false)
{
  $dev = $wifi ? "wlan0" : "eth0";
  $res = shell_exec ("ip addr show $dev | grep -E 'inet [0-9]' | awk '{print $2}'");
  return CreateNetmaskAddr((int) explode ("/", $res)[1]);
}

function GetGateway($wifi = false)
{
  $dev = $wifi ? "wlan0" : "eth0";
  $res = shell_exec("ip route list | grep 'default' | grep '$dev' | awk '{print $3}'");
  $res = trim($res);
  if ($res)
    return $res === $dev ? "0.0.0.0" : $res;
  return "0.0.0.0";
}

function GetDNS()
{ // predelat na resolvconf ??
  $res = shell_exec("cat /etc/resolv.conf | grep 'nameserver' | awk '{print $2}'");
  $first = explode("\n", $res)[0];
  $first = trim($first);
  return $first ? $first : "0.0.0.0";
}

function GetPort()
{
  global $astro;
  $port = $astro ? "as_port" : "dc_port";
  $res = shell_exec("cat /etc/micamd.conf | grep -E $port | awk '{print $3}'");
  return trim($res);
}

function GetSerialPort($pos = 0)
{
  $port = $pos == 0 ? "serial_port_a" : "serial_port_b";
  $res = shell_exec("cat /etc/micamd.conf | grep -E $port | awk '{print $3}'");
  return trim($res);
}

function GetMAC($wifi = false)
{
  $dev = $wifi ? "wlan0" : "eth0";
  $res = shell_exec("ip addr show $dev | grep ether | awk '{print $2}'");
  return trim($res);
}

function GetVersionNum()
{
  $ver = GetFromCli("version");
  return (int) trim(explode("-", $ver)[0]);
}

function GetVersionStr()
{
  $ver = GetFromCli("version");
  return trim(explode("-", $ver)[1]);
}

function Update()
{
  global $err, $msg;

  $ret = SendToCli("update");
  if ($ret == 0) {
    $msg = "Firmware was successfully updated.";
    return true;
  } else {
    $err = "An error occurred during firmware update. Exit code: " . $ret;
    return false;
  }
}

function TryUploadUpdate()
{
  global $err;

  if ((!empty($_FILES["userfile"])) && ($_FILES["userfile"]["error"] == 0)) {
    $newname = TMP_LOCATION . "/" . UPD_FILE_NAME;
    if (move_uploaded_file($_FILES["userfile"]["tmp_name"], $newname)) {
      Update();
      return;
    } else {
      $err = "An error occurred during file moving.";
    }
    return;
  }

  $err = "An error occurred during file upload.";
}

function TryWebUpdate()
{
  global $err, $msg, $astro;
  $url = $astro ? "http://www.gxccd.com/gxetha/" : "http://register.mii.cz/datacameth/";
  $arm = IsARM();
  $file = $arm ? "version_arm.txt" : "version.txt";

  $content = file_get_contents($url . $file);
  if ($content === FALSE) {
    $err = "Could not get firmware version. Connection to update server failed.";
    return;
  }

  $content = explode ("\n", $content);
  $wfw = (int) trim ($content[0]);
  $myfw = GetVersionNum();
  $md5sum = trim ($content[2]);

  if ($myfw < $wfw) {
    if ($arm) {
      $fw = $url . "arm/" . trim ($content[1]);
    } else {
      $fw = $url . trim ($content[1]);
    }

    $content = file_get_contents ($fw);
    if ($content === FALSE) {
      $err = "Couldn't download new firmware. Connection to update server failed or update file doesn't exist.";
      return;
    }

    $name = TMP_LOCATION . "/" . UPD_FILE_NAME;
    // write file
    $file = fopen ($name, "wb");
    fwrite ($file, $content);
    fclose ($file);
    // check md5
    if (md5_file ($name) === $md5sum) {
      Update ();
    } else {
      $err = "Firmware check failed. The downloaded file is corrupted.";
    }
  } else {
    $msg = "Your firmware is up-to-date.";
  }
}

function ProcessForm()
{
  global $err, $msg, $redirect, $astro;
  $netreset = false;
  $dreset = false;
  $nmset = false;

  $ip = $nm = $gw = $dns = "";
  if (IsLANConnected())
  {
    $ip  = $_POST["a0"] . "." . $_POST["a1"] . "." . $_POST["a2"] . "." . $_POST["a3"];
    $nm  = $_POST["n0"] . "." . $_POST["n1"] . "." . $_POST["n2"] . "." . $_POST["n3"];
    $gw  = $_POST["g0"] . "." . $_POST["g1"] . "." . $_POST["g2"] . "." . $_POST["g3"];
    $dns = $_POST["d0"] . "." . $_POST["d1"] . "." . $_POST["d2"] . "." . $_POST["d3"];
  }
  $port = $_POST["po"];
  $sportA = 0;
  $sportB = 0;
  if ($astro)
  {
    $sportA = $_POST["spa"];
    if (!IsARM())
      $sportB = $_POST["spb"];
  }
  $hostname = trim($_POST["hn"]);

  $res = true;
  if (IsLANConnected())
    $res = CheckAddress($ip) && CheckAddress($nm) && CheckAddress($gw) && CheckAddress($dns);

  $res &= is_numeric($port) && is_numeric($sportA) && is_numeric($sportB);

  if ($res == false)
  {
    $err = "Invalid address or ports.";
    return;
  }

  $res = 0;
  if (IsLANConnected())
  {
    $cidr = Netmask2Cidr($nm);
    $networkip = Cidr2Network($ip, $cidr);
    if (CidrMatchIP($gw, $networkip, $cidr) == false)
    {
      $err = "Gateway IP address is not in the submitted subnet.";
      return;
    }

    if ($ip != GetIP())
    {
      $res |= SendToCli("setipnm eth " . $ip . "\\\/" . Netmask2Cidr ($nm));
      if ($res == 0)
      {
        $netreset = true;
        $redirect = $ip;
        $nmset = true;
      }
    }
    if ($nm != GetNetmask() && !$nmset)
    {
      $res |= SendToCli("setipnm eth " . $ip . "\\\/" . Netmask2Cidr ($nm));
      if ($res == 0)
        $netreset = true;
    }
    if ($gw != GetGateway())
    {
      $res |= SendToCli("setgw eth " . $gw);
      if ($res == 0)
        $netreset = true;
    }
    if ($dns != GetDNS())
    {
      $res |= SendToCli("setdns eth " . $dns);
      if ($res == 0)
        $netreset = true;
    }
  }
  if ($port != GetPort())
  {
    $res |= SendToCli("setport " . $port);
    if ($res == 0)
      $dreset = true;
  }
  if ($hostname != gethostname())
  {
    $res |= SendToCli("sethn " . $hostname);
  }

  if ($astro)
  {
    if ($sportA != GetSerialPort(0))
    {
      $res |= SendToCli("setsport a " . $sportA);
      if ($res == 0)
        $dreset = true;
    }
    if (!IsARM()) {
      if ($sportB != GetSerialPort(1))
      {
        $res |= SendToCli("setsport b " . $sportB);
        if ($res == 0)
          $dreset = true;
      }
    }
  }

  if ($res == 0) {
  	if (!empty($redirect)) {
      exec("sh -c '(sleep 2; micamd-cli restart net) &' >/dev/null");
    }
    if ($netreset && empty($redirect)) {
      SendToCli ("restart net");
      sleep(5);
    }
    if ($dreset) {
      SendToCli ("restart daemon");
    }

    $msg = "Configuration updated successfully.";
  } else {
    $err = "An error occurred during configuration update.";
  }
}

function CreateNetmaskAddr($bitcount)
{
  $netmask = str_split(str_pad(str_pad("", $bitcount, "1"), 32, "0"), 8);
  foreach ($netmask as &$element) $element = bindec($element);
  return trim(join(".", $netmask));
}

function Netmask2Cidr($mask)
{
   $long = ip2long($mask);
   $base = ip2long("255.255.255.255");
   return 32 - log(($long ^ $base) + 1, 2);
}

function Cidr2Network($ip, $cidr)
{
  return long2ip((ip2long($ip)) & ((-1 << (32 - (int) $cidr))));
}

function CidrMatchIP($ip, $network, $cidr)
{
  return ((ip2long($ip) & ~((1 << (32 - $cidr)) - 1)) == ip2long($network));
}

function CheckAddress($addr)
{
  $a = explode(".", $addr);
  foreach ($a as $part) {
    if (!(is_numeric($part)))
      return false;
  }
  return true;
}

function SendToCli($cmd)
{
  $out = array();
  $ret = 0;
  exec("micamd-cli " . $cmd . " 2>&1", $out, $ret);
  if ($ret) {
    $f = fopen("/tmp/errors", "a+");
    fwrite($f, ">>---------- micamd-cli $cmd ----------\n");
    if (is_array($out)) {
      foreach ($out as $s) {
        fwrite($f, $s . "\n");
        error_log($s);
      }
    } else {
      fwrite($f, $out);
      error_log($out);
    }
    fwrite($f, "<<-----------------------------------------\n\n");
    fclose($f);
  }
  return $ret;
}

function GetFromCli($cmd)
{
  return shell_exec("micamd-cli " . $cmd . " 2>>/tmp/errors");
}

///////////////////////////////////////////////////////////////////////////////
// WLAN functions
///////////////////////////////////////////////////////////////////////////////

function HasWLAN()
{
  $res = shell_exec("ip addr show | grep 'wlan' | wc -l");
  $wlan = trim($res) !== "0";
  if ($wlan) {
    if (!WLANGetStatusAndESSID()["conn"]) {
      SendToCli("scan-wifi");
      WLANUpdateNetworkList();
    }
  }
  return $wlan;
}

function WLANGetStatusAndESSID()
{
  $res = shell_exec("iwconfig wlan0 | grep 'Not-Associated' | wc -l");
  $conn = trim($res) === "0";
  $ssid = "";
  if ($conn) {
    $ssid = shell_exec("iwconfig wlan0 | head -1 | grep -Eo 'ESSID:\\\"(.+)\\\" ' | cut -d: -f2");
    $ssid = trim(trim($ssid), "\"");
  }
  return array(
    "conn" => $conn,
    "ssid" => $ssid
  );
}

function WLANdBmToQuality($dBm)
{
  if ($dBm <= -100)
    return 0;
  else if ($dBm >= -50)
    return 100;
  else
    return 2 * ($dBm + 100);
}

function WLANUpdateNetworkList()
{
  $ssidList = shell_exec('iwlist wlan0 scan last | gawk \'BEGIN { i = -1 }; /Cell/ { i++ }; /ESSID/ { split($0, b, ":"); a[i][1] = b[2] }; /Encryption/ { split($2, b, ":"); a[i][2] = b[2] }; /WPA/ { if (!index(a[i][3], "WPA")) a[i][3] = "WPA" } /Signal/ { split($3, b, "="); a[i][4] = b[2] }; END { for (i in a){ s=""; for (j=1; j<5;j++) s = s a[i][j] "\0"; print s } }\'');
  $_SESSION['ssidList'] = explode("\n", trim($ssidList));
}

function WLANGetNetworkList()
{
  $a = array();
  foreach ($_SESSION['ssidList'] as $info) {
    $items = explode(chr(0), $info);
    $key = trim($items[0], "\"'");
    $a[$key] = array();
    $a[$key]['enc'] = trim($items[1]) == "on";
    $a[$key]['wpa'] = $items[2] == "WPA";
    $a[$key]['signal'] = WLANdBmToQuality((int) $items[3]);
  }
  ksort($a, SORT_STRING | SORT_FLAG_CASE);
  return $a;
}

function ProcessWifiForm($connect)
{
  if (!$connect) {
    SendToCli("stop-wifi");
    return;
  }

  global $err, $msg;
  $ssid = $_POST["essid"];
  $pass = $_POST["essid-pass"];
  $type = $_POST["wifi-ip-type"];
  $ip   = $_POST["wifi-a0"] . "." . $_POST["wifi-a1"] . "." . $_POST["wifi-a2"] . "." . $_POST["wifi-a3"];
  $nm   = $_POST["wifi-n0"] . "." . $_POST["wifi-n1"] . "." . $_POST["wifi-n2"] . "." . $_POST["wifi-n3"];
  $gw   = $_POST["wifi-g0"] . "." . $_POST["wifi-g1"] . "." . $_POST["wifi-g2"] . "." . $_POST["wifi-g3"];
  $dns  = $_POST["wifi-d0"] . "." . $_POST["wifi-d1"] . "." . $_POST["wifi-d2"] . "." . $_POST["wifi-d3"];

  if ($type == "static") {
    if ($gw === "...") {
      $gw = "0.0.0.0";
    } else if ($gw !== "0.0.0.0") {
      $cidr = Netmask2Cidr($nm);
      $networkip = Cidr2Network($ip, $cidr);
      if (!CidrMatchIP($gw, $networkip, $cidr)) {
        $err = "Gateway IP address is not in the submitted subnet.";
        return;
      }
    }

    if ($dns === "...")
      $dns = "0.0.0.0";
  }

  $res = 0;
  $ssidList = WLANGetNetworkList();
  $isWpa = $ssidList[$ssid]['enc'] && $ssidList[$ssid]['wpa'];
  if ($ssidList[$ssid]['enc']) {
    if ($isWpa)
      $res |= SendToCli("setsec wpa");
    else
      $res |= SendToCli("setsec wep");
  } else {
    $res |= SendToCli("setsec none");
  }

  $res |= SendToCli("setssid " . escapeshellarg($ssid));

  // if (!$isWpa && ctype_xdigit($pass)) {
  //   $pass = escapeshellarg('\"' . $pass);
  // } else {
    $pass = escapeshellarg($pass);
  //}
  if ($ssidList[$ssid]['enc'])
    $res |= SendToCli("setpw " . $pass);

  if ($type == "static") {
    $res |= SendToCli("setipnm wifi " . $ip . "\\\\/" . Netmask2Cidr($nm));

    if ($gw === "0.0.0.0")
      $res |= SendToCli("comment gw wifi");
    else
      $res |= SendToCli("setgw wifi $gw");

    if ($dns === "0.0.0.0")
      $res |= SendToCli("comment dns wifi");
    else
      $res |= SendToCli("setdns wifi $dns");
  }

  $res |= SendToCli("start-wifi $type");

  if ($res == 0) {
    $msg = "Wi-Fi connection established successfully.";
  } else {
    $err = "Wi-Fi connection failed. Please check your password and/or IP settings.";
  }
}

const astro_enumHeaderSize = 8 + 4 + 4 + 4 + 4;
const astro_cmdMagic1 = 2696004307;
const astro_cmdMagic2 = 3841261569;
const astro_resMagic1 = -1314914303;
const astro_resMagic2 = -169552958;
const astro_infoSize = 66 + 1;

function EnumerateCamerasAstro()
{
  $port = GetPort();

  $sock = stream_socket_client("localhost:$port", $errno, $errstr);
  if ($sock === false) {
    return false;
  }
  $data = pack('LLLL', astro_cmdMagic1, astro_cmdMagic2, 0, 999);
  if (fwrite($sock, $data) === false) {
    fclose($sock);
    return false;
  }
  $data = fread($sock, astro_enumHeaderSize);
  if ($data === false) {
    fclose($sock);
    return false;
  }
  $data = unpack('L2magic/Lsize/Lcmd/Lres/Lnum_cams', $data);
  if ($data === false) {
    fclose($sock);
    return false;
  }
  if ($data['magic1'] !== astro_resMagic1 || $data['magic2'] !== astro_resMagic2 || $data['res'] !== 0 || $data['cmd'] !== 999) {
    fclose($sock);
    return false;
  }
  if ($data['num_cams'] === 0) {
    fclose($sock);
    return false;
  }
  $cams = fread($sock, $data['size'] - astro_enumHeaderSize);
  if ($cams === false) {
    fclose($sock);
    return false;
  }
  fclose($sock);
  $result = array();
  for ($i = 0; $i < $data['num_cams']; $i++) {
    $cam = unpack('Lid/Z13tmp1/Z15desc/Z15serial/Z19tmp2/Cstatus', $cams);
    if ($cam === false) {
      return false;
    }
    $result[] = array(
      'id' => $cam['id'],
      'desc' => trim($cam['desc']),
      'serial' => trim($cam['serial']),
      'status' => (int) $cam['status'],
    );
    $cams = substr($cams, astro_infoSize);
  }
  return $result;
}

const dc_enumHeaderSize = 4 + 4 + 4 + 4;
const dc_cmdMagic = 623125312;
const dc_resMagic = 623125312;
const dc_infoSize = 59 + 1;

function EnumerateCamerasDC()
{
  $port = GetPort();

  $sock = stream_socket_client("localhost:$port", $errno, $errstr);
  if ($sock === false) {
    return false;
  }
  $data = pack('LLL', dc_cmdMagic, 0, 999);
  if (fwrite($sock, $data) === false) {
    fclose($sock);
    return false;
  }
  $data = fread($sock, dc_enumHeaderSize);
  if ($data === false) {
    fclose($sock);
    return false;
  }
  $data = unpack('Lmagic/Lid/Lcmd/Lnum_cams', $data);
  if ($data === false) {
    fclose($sock);
    return false;
  }
  if ($data['magic'] !== dc_resMagic || $data['id'] !== 0 || $data['cmd'] !== 999) {
    fclose($sock);
    return false;
  }
  if ($data['num_cams'] === 0) {
    fclose($sock);
    return false;
  }
  $cams = fread($sock, $data['num_cams'] * dc_infoSize);
  if ($cams === false) {
    fclose($sock);
    return false;
  }
  fclose($sock);
  $result = array();
  for ($i = 0; $i < $data['num_cams']; $i++) {
    $cam = unpack('Lid/Z10tmp1/Z15desc/Z15serial/Z15tmp2/Cstatus', $cams);
    if ($cam === false) {
      return false;
    }
    $result[] = array(
      'id' => $cam['id'],
      'desc' => trim($cam['desc']),
      'serial' => trim($cam['serial']),
      'status' => (int) $cam['status'],
    );
    $cams = substr($cams, dc_infoSize);
  }
  return $result;
}
